package jalview.datamodel;

public class AlignmentAnnotation
{
  public String label;
  public String description;
  public Annotation  [] annotations;
  public boolean isGraph = false;
  public float graphMin, graphMax;
  public int windowLength;

  // Graphical hints and tips
  public boolean editable = false;
  public boolean hasIcons; //
  public boolean hasText;
  public boolean visible = true;
  public int height=0;

  public AlignmentAnnotation(String label, String description, Annotation [] annotations)
  {
    // always editable?
    editable = true;
    this.label = label;
    this.description = description;
    this.annotations = annotations;
    for(int i=0; i<annotations.length; i++)
    {
      if (annotations[i]!=null &&(
          annotations[i].secondaryStructure == 'H' ||
          annotations[i].secondaryStructure == 'E'))
        hasIcons = true;

      if (annotations[i]!=null && annotations[i].displayCharacter.length()>0)
        hasText = true;
    }
  }

  public AlignmentAnnotation(String label, String description, Annotation [] annotations, float min, float max, int winLength)
  {
    // graphs are not editable
    this.label = label;
    this.description = description;
    this.annotations = annotations;
    isGraph = true;
    if(min==max)
    {
      for(int i=0; i<annotations.length; i++)
      {
        if(annotations[i]==null)
          continue;
        if(annotations[i].value>max)
          max = annotations[i].value;
        if(annotations[i].value<min)
          min = annotations[i].value;
      }
    }

    graphMin = min;
    graphMax = max;
    windowLength = winLength;
    for(int i=0; i<annotations.length; i++)
    {
      if (annotations[i]!=null &&(
          annotations[i].secondaryStructure == 'H' ||
          annotations[i].secondaryStructure == 'E'))
        hasIcons = true;

      if (annotations[i]!=null && annotations[i].displayCharacter.length()>0)
        hasText = true;
    }
  }

  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    for(int i=0; i<annotations.length; i++)
   {
     buffer.append( annotations[i].value );
     buffer.append(", ");
   }
   return buffer.toString();
  }

}
